#include "file_extractor.h"

char* logFileName = "quickbms_output.txt";

void PAZExtract(int menuselectedOption, char* filter, char* extractDir)
{
    FILE* metaFile = NULL;  // The pad00000.meta file (We are going to use it to obtain how many .PAZ files we have)
    int pazCount = 0;      // How many .PAZ files we have
    char command[MAX_PATH];

    // Opens the pad00000.meta
    metaFile = openFile("pad00000.meta","rb");
    fseek(metaFile,sizeof(int),SEEK_SET); // Skips the first in in the meta file since it's not relevant for this operation
    fread(&pazCount,sizeof(int),1,metaFile); // The second int file in the meta file, contains how many .PAZ file the game has
    fclose(metaFile);

    // For all files
    int currentFileNum = 1;
    for(currentFileNum = pazCount; currentFileNum >= 1; currentFileNum--)
    {
        if (menuselectedOption == 1)
        {
             sprintf(command,"quickbms.exe -Y 1>>%s blackdesert.bms PAD%.5d.PAZ \"%s",logFileName, currentFileNum, extractDir);
        }
        else if (menuselectedOption != 6)
        {
            sprintf(command,"quickbms.exe -Y -f \"*%s\" 1>>%s blackdesert.bms PAD%.5d.PAZ \"%s", filter,logFileName, currentFileNum, extractDir);
        }
        if (menuselectedOption == 6)
        {
            printf("\nSearching for: %s\n", filter);

            sprintf(command,"quickbms.exe -Y -f \"*%s*\" 1>>%s blackdesert.bms PAD%.5d.PAZ \"%s", filter, logFileName, currentFileNum, extractDir);
            printf("commmand: %s\n", command);
            system(command);

            printf("\nCurrent File: PAD%.5d.PAZ\n", currentFileNum); // Show which file we are processing

            if (!logFileIsEmpty())
            {
                printf("Found and extracted!");
                break;
            }
        }
        else
        {
            system(command);
            printf("commmand: %s\n", command);
            printf("\nCurrent File: PAD%.5d.PAZ\n", currentFileNum); // Show which file we are processing
            printf("(%d Percent Complete)\n", calculateProgress(currentFileNum,pazCount));

            if(menuselectedOption == 1 && logFileIsEmpty())
            {
                printf("\n\nSomething went wrong, could not extract any files.\n\n");
                break;
            }
        }
    }
}

char* getPAZName(char* fileName)
{
    FILE* pazRefFile = openFile("pazNamesRelations.txt","rb");
    char* pazReferences = copyFileToMemory(pazRefFile)->bytes;
    fclose(pazRefFile);

    char* fileNameRead = strtok(pazReferences,"|");
    char* PAZNameRead = NULL;
    if (fileNameRead != NULL)
    {
         PAZNameRead = strtok(NULL,"\n");
    }

    while (fileNameRead != NULL)
    {
        if (strcmpi(fileNameRead,fileName) == 0)
        {
            return PAZNameRead;
        }

        fileNameRead = strtok(NULL,"|");
        if (fileNameRead != NULL)
        {
            PAZNameRead = strtok(NULL,"\n");
        }
        else
        {
            break;
        }
    }
    return "";
}


// Uses quickbms commands to extract the textures in the specified folder "extractDir"
void extractFiles(int menuselectedOption, char* filter, char* extractDir)
{
    FILE* logFile = NULL;
    char command[MAX_PATH] = "";   // The actual quickbms command that is going to be run in the command prompt

    // Creates an empty log file
    logFile = openFile("quickbms_output.txt","wb");
    fclose(logFile);

    if(menuselectedOption == 6)
    {
        char* PAZName = getPAZName(filter);
        if (strlen(PAZName) > 0)
        {
            sprintf(command,"quickbms.exe -Y -f \"*%s*\" 1>quickbms_output.txt blackdesert.bms %s \"%s", filter,PAZName, extractDir);
            printf("commmand: %s\n", command);
            system(command);
        }
        else
        {
            PAZExtract(menuselectedOption,filter,extractDir);
        }
    }
    else if (menuselectedOption == 7)
    {
        sprintf(command,"quickbms.exe -Y 1>quickbms_output.txt blackdesert.bms pad00000.meta \"%s", extractDir);
        printf("\ncommmand: %s\n", command);

        system(command);
    }
    else if (menuselectedOption == 8)
    {
        sprintf(command,"quickbms.exe -Y 2>NULL 1>quickbms_output.txt blackdesert.bms pad00000.meta \"%s", extractDir);
        printf("commmand: %s\n", command);

        printf("\nThis will take a while, don't worry, you will see a \"Finished\" message once it's done.");
        printf("\nTo see if the program is doing something,\nKeep pressing F5 in your PAZ folder and see if the file \"quickbms_output.txt\" is increasing size\n");
        printf("\nExtracting files...\n");
        system(command);
        remove("NULL");

        if (logFileIsEmpty())
        {
            sprintf(command,"quickbms.exe -Y blackdesert.bms pad00000.meta \"%s", extractDir);
            system(command);
        }
    }

    if(menuselectedOption < 6) // For all files extraction of extension extraction
    {
         PAZExtract(menuselectedOption,filter,extractDir);
    }

    if (!logFileIsEmpty())
    {
        printf("\n\nFinished!\n\n");
    }
    else
    {
        printf("\n\n Error! no files extracted \n\n");
    }
}


    // 2>NULL suppresses the default quickbms output
    // >quickbms_output.txt makes all the extraction output that quickbms generates to be written in a file
    // -Y is automatic "yes for all"
    // -f "*.dds" Fiters so only .dds files are extracted
    // blackdesert.bms the Black Desert quickbms plugin file



int logFileIsEmpty()
{
    FILE* logFile = openFile(logFileName,"rb");
    if (getFileSize(logFile) > 0)
    {
        fclose(logFile);
        return 0;
    }
    fclose(logFile);
    return 1;
}

// Does a simple 3-way rule to calculate the program completeness percentage
int calculateProgress(int currentPazNun, int pazCount)
{
    return ((int)((double)((pazCount-currentPazNun)*100.0)/(double)pazCount));
}
